/*
 * unclutter: remove idle cursor image from screen so that it doesnt
 * obstruct the area you are looking at.
 * doesn't do it if cursor is in root window or a button is down.
 * Tries to cope with jitter if you have a mouse that twitches.
 * (Could be a little fancier and only do it in particular clients.)
 * Unfortunately, clients like xterm and emacs set different text cursor
 * shapes depending on whether they have pointer focus or not.
 * Mark M Martin. cetia 1991  mmm@cetia.fr
 */
#include <X11/Xos.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdio.h>

char *progname;
pexit(str)char *str;{
    fprintf(stderr,"%s: %s\n",progname,str);
    exit(1);
}
usage(){
    pexit("usage:\n\
	-display <display>\n\
	-idle <seconds>		time between polls to detect idleness.\n\
	-jitter <pixels>	pixels mouse can twitch without moving\n\
 	-root	       		apply to cursor on root window too");
}

#define ALMOSTEQUAL(a,b) (abs(a-b)<=jitter)
#define ANYBUTTON (Button1Mask|Button2Mask|Button3Mask|Button4Mask|Button5Mask)

main(argc,argv)char **argv;{
    Display *display;
    Cursor cursor;
    int status,screen,oldx,oldy = -99;
    int doroot = 0, jitter = 0, idletime = 5;
    Window root;
    Pixmap cursormask;
    XGCValues xgc;
    GC gc;
    XColor dummycolour;
    char *displayname = 0;
    
    progname = *argv;
    argc--;
    while(argv++,argc-->0){
	if(strcmp(*argv,"-idle")==0){
	    argc--,argv++;
	    if(argc<0)usage();
	    idletime = atoi(*argv);
	}else if(strcmp(*argv,"-jitter")==0){
	    argc--,argv++;
	    if(argc<0)usage();
	    jitter = atoi(*argv);
	}else if(strcmp(*argv,"-root")==0){
	    doroot = 1;
	}else if(strcmp(*argv,"-display")==0){
	    argc--,argv++;
	    if(argc<0)usage();
	    displayname = *argv;
	}else usage();
    }
    display = XOpenDisplay(displayname);
    if(display==0)pexit("could not open display");
    screen = DefaultScreen(display);
    root = RootWindow(display,screen);

    cursormask = XCreatePixmap(display, root, 1, 1, 1/*depth*/);
    xgc.function = GXclear;
    gc =  XCreateGC(display, cursormask, GCFunction, &xgc);
    XFillRectangle(display, cursormask, gc, 0, 0, 1, 1);
    dummycolour.pixel = 0;
    dummycolour.red = 0;
    dummycolour.flags = 04;
    cursor = XCreatePixmapCursor(display, cursormask, cursormask,
	      &dummycolour,&dummycolour, 0,0);
    XFreePixmap(display,cursormask);
    XFreeGC(display,gc);

    while(1){
	Window dummywin,windowin;
	int x,y,dummyxy;
	unsigned int modifs;
	
	if(XQueryPointer(display, root, &dummywin, &windowin,
			 &x, &y, &dummyxy, &dummyxy, &modifs)){
	    if((!doroot && windowin==None) || (modifs & ANYBUTTON) ||
	       !(ALMOSTEQUAL(x,oldx) && ALMOSTEQUAL(y,oldy))){
		oldx = x, oldy = y;
		sleep(idletime);


	    }else{
		status =  XGrabPointer(display, root, 0,
		       PointerMotionMask|ButtonPressMask|ButtonReleaseMask,
		       GrabModeAsync, GrabModeAsync, None, cursor,
		       CurrentTime);
		if(status==GrabSuccess){
		    XEvent event;
		    do{
			XNextEvent(display,&event);
		    }while(event.type==MotionNotify &&
			   ALMOSTEQUAL(x,event.xmotion.x) &&
			   ALMOSTEQUAL(y,event.xmotion.y));
		    XUngrabPointer(display, CurrentTime);
		}
	    }
	}
    }
}
